#!/bin/bash
# encoding: utf-8

lang=de
tstyle='+%d.%m.%Y_%H:%M'
useroot='Bitte rufen Sie dieses Script mit Root-Rechten auf!'
nodesinfect="Es wurde kein Laufwerk mit Desinfec't gefunden."
twodesinfect="Es wurden mehrere Laufwerke mit Desinfec't gefunden."
alldone="Starten Sie erneut vom USB-Stick und wählen Sie im Bootmenü den Eintrag \"Desinfec't von USB-Stick entfernen\"."
case $LANG in 
	nl*)
		lang=nl
		tstyle='+%d-%m-%Y_%H:%M'
		useroot='Roep dit script op met root-rechten.'
		nodesinfect="Er is geen medium met Desinfec't gevonden."
		twodesinfect="Er zijn meerdere media met Desinfec't gevonden."
		alldone="Start opnieuw op van de usb-stick en selecteer in het bootmenu \"Desinfec't van usb-stick verwijderen\"."
	;;
esac

me=` id -u `
if [ "$me" -gt 0 ] ; then
	echo '***> '"${useroot}"
	zenity --error --text "$useroot"
	exit 1
fi

# Identifiziere den Stick
STICK=""
STICKCOUNT=0
ISOHYBRID=0

# Identify properly partitioned drives: 
for d in a b c d e f g h i j k l m n o ; do
	for e in vd xvd sd ; do
		if [ -b /dev/${e}${d}3 ] ; then
			label=` blkid -o udev /dev/${e}${d}3 | grep ID_FS_LABEL_ENC | awk -F '=' '{print $2}' `
			if [ "$label" = desinfSIGS ] ; then
				STICK="/dev/${e}${d}"
				STICKCOUNT=` expr $STICKCOUNT + 1 ` 
			fi
		fi
	done
done
if [ "$STICKCOUNT" -lt 1 ] ; then
	for d in a b c d e f g h i j k l m n o ; do
		for e in vd xvd sd ; do
			if [ -b /dev/${e}${d} ] ; then
				label=` blkid -o udev /dev/${e}${d} | grep ID_FS_LABEL_ENC | awk -F '=' '{print $2}' `
				fstype=` blkid -o udev /dev/${e}${d} | grep ID_FS_TYPE | awk -F '=' '{print $2}' `
				if [ "$label" = DESINFECT -a "$fstype" = iso9660 ] ; then
					STICK="/dev/${e}${d}"
				fi
			fi
		done
	done
	# We are running from an unchanged ISOhybrid drive, do not care, just kexec...
	if [ -n "$STICK" ] ; then
		for n in ` seq 1 10 ` ; do
			cat /proc/mounts | grep ntfs | awk '{print $2}' | while read d 
			do
				umount "$d"
			done
			cat /proc/mounts | grep fat | awk '{print $2}' | while read d 
			do
				umount "$d"
			done
			cat /proc/mounts | awk '{print $2}' | grep '/media' | while read d 
			do
				umount "$d"
			done
		done
		forcekexec=1
		if cat /proc/mounts | grep /media ; then 
			if zenity --question --text 'There are still mounted drives. Uninstalling might cause damage to open files. Do you want to uninstall anyway?' ; then
				forcekexec=1
			else
				forcekexec=0
			fi
		else
			if zenity --question --text 'Wollen Sie die Deinstallation unmittelbar starten (geöffnete Programme werden sofort geschlossen, nicht gespeicherte Daten gehen verloren)?' ; then
				forcekexec=1
			else
				forcekexec=0
			fi
		fi
		if [ "$forcekexec" -gt 0 ] ; then
			install -m 0755 /usr/share/desinfect-remaster/initramfs-stretch/sbin/kexec /usr/sbin
			chvt 1
			/usr/sbin/kexec -f --initrd=/cdrom/casper/initrd.str --append='quiet uninstall=23 nonet' -x /cdrom/casper/vmlinuz
		fi 
		# If kexec fails (UEFI), we'll continue:
	fi
fi

if [ "$STICKCOUNT" -lt 1 ] ; then
	zenity --error --text "$nodesinfect"
	exit 1
fi

if [ "$STICKCOUNT" -gt 1 ] ; then
	zenity --error --text "$twodesinfect"
	exit 1
fi

mkdir -p /desinfect/boot
mount /dev/disk/by-label/desinfDATA /desinfect/boot
sed -i 's%#zap#%%g' /desinfect/boot/boot/grub/grub.cfg 
sed -i 's%#zap#%%g' /desinfect/boot/isolinux/os.cfg

zenity --info --text "$alldone"
