#!/bin/bash
# encoding: utf-8

YEAR=202021
ARCH=amd64

if [ `id -u ` -gt 0 ] ; then
	echo "Please run as root. Exiting."
	exit 1
fi
if mountpoint -q /desinfect/target ; then
	echo "Directory /desinfect/target is a mount point. Exiting."
	exit 1
fi
if [ -z "$1" ] ; then
	echo "Please specify target device."
	exit 1
fi
if [ -b "$1" ] ; then
	echo "Target is block device, installing..."
else
	echo "Target must be a block device"
	exit 1
fi
if [ -d /cdrom/casper/filesystem.dir ] ; then
	echo "Install from BTRFS source is not yet supported."
	exit 1
fi
if [ -f /isodevice/software/desinfect-${YEAR}-${ARCH}.iso ] ; then
	echo "Found desinfect-${YEAR}-${ARCH}.iso, installing..."
else
	echo "Please run from outer DVD."
	exit 1
fi
mkdir -p /desinfect/target

TARGET="$1"

dd if=/dev/zero bs=1M count=8 of=${TARGET} 
parted -s $TARGET mklabel msdos
parted -s $TARGET mkpart primary fat32 1M 100%
parted -s $TARGET set 1 boot on
parted -s $TARGET set 1 esp on
dd if=/usr/lib/syslinux/mbr/mbr.bin of=${TARGET} 
mkfs.msdos -F32 ${TARGET}1
mkdir -p /desinfect/target
mount -o rw ${TARGET}1 /desinfect/target
tar -C /cdrom -cvf - isolinux | tar -C /desinfect/target -xf -
extlinux -s -i /desinfect/target/isolinux
echo "Syncing..."
sync
echo "Do you want to include the SquashFS container? When answering \"n\" the thumb"
echo "drive can only be used to kickstart Desinfec't on a regular thumb drive [y/n]"
read choice
case $choice in
	y*)
		tar -C /cdrom -cvf - \
		boot casper EFI 'Desinfect Hilfe' LIESMICH.htm LIESMICH.txt \
		'TeamViewer Portable' dvdcover.pdf ct.ico | tar -C /desinfect/target -xf -
	;;
	*)
		tar -C /cdrom --exclude=filesystem.squashfs -cvf - \
		boot casper EFI 'Desinfect Hilfe' LIESMICH.htm LIESMICH.txt \
		'TeamViewer Portable' dvdcover.pdf ct.ico | tar -C /desinfect/target -xf -
	;;
esac

cp -v  /desinfect/target/isolinux/isolinux.cfg /desinfect/target/isolinux/syslinux.cfg
sed -i 's%include local.cfg%#include local.cfg%g' /desinfect/target/isolinux/syslinux.cfg
sed -i 's%#menu default%menu default%g' /desinfect/target/isolinux/os.cfg

echo "Do you want to replace GRUB with Gummmiboot as UEFI boot loader? Please note" 
echo "that while offering better compatibility, it is less comfortable. [y/n]"
read choice
case $choice in
	y*)
		tar -C /opt/desinfect/usb-installer/gummiboot -cvf - EFI loader | tar -C /desinfect/target -xf -
	;;
esac

echo "Syncing..."
sync
# rsync -avHP --inplace /cdrom/software/ /desinfect/target/software/
echo "Syncing..."
sync
umount /desinfect/target
echo "Done."
