#!/usr/bin/ruby
# encoding: utf-8

require 'glib2'
require 'gtk2'
require 'vte'
require "rexml/document"
require 'MfsDiskDrive'
require 'MfsSinglePartition'
require 'optparse' 
require 'fileutils'

LOCSTRINGS = {
	"de" => {
		"nodrive" => "Kein geeignetes Laufwerk gefunden",
		"reallywant" => "Wollen Sie wirklich installieren?",
		"reallydetails" => "Wollen Sie Desinfec't wirklich auf dem USB-Laufwerk MODEL VENDOR SIZE installieren?\n\n<b>Hierbei werden alle auf dem Laufwerk gespeicherten Daten gelöscht!</b>",
		"reallyresize" => "Wollen Sie Desinfec't wirklich auf dem USB-Laufwerk MODEL VENDOR SIZE installieren? Haben Sie das LIESMICH zu diesem Programm gelesen?\n\n<b>Trotz NTFS-Größenänderung besteht das Risiko eines vollständigen Datenverlustes!</b>",
		"largedrive" => "<span size='large' weight='bold'>ACHTUNG!</span>\n\nDer ausgewählte Datenträger VENDOR MODEL  ist\n\n<span size='large' weight='bold'>SIZE</span>\n\ngroß und soll komplett gelöscht werden. Sind Sie wirklich ganz sicher, dass es sich nicht um eine Festplatte handelt, die Sie noch brauchen?\n\nKlicken Sie auf \"Ja\", um das Laufwerk zu löschen und Desinfec't zu installieren, mit \"Nein\" gelangen Sie zurück zur Laufwerksauswahl.",
		"progress" => "Installationsfortschritt",
		"installing" => "Die Installation wird durchgeführt, bitte ziehen Sie den USB-Stick während der Installation nicht ab.",
		"installtitle" => "Die Installation ist abgeschlossen",
		"installdone" => "Die Installation ist abgeschlossen, Sie können den USB-Stick jetzt abziehen.",
		"installdone2" => "Die Installation ist abgeschlossen, der USB-Stick wird nun verwendet und darf nicht abgezogen werden.",
		"delstick" => "Lösche USB-Stick...",
		"delpart" => "Lösche Partition NUM...",
		"delmbr" => "Lösche MBR...",
		"delgpt" => "Lösche Backup-GPT...",
		"mklabel" => "Erstelle Partitionstabelle...",
		"mkpart" => "Erstelle Partition NUM...",
		"ptoolow" => "Der Partitionsbeginn der ersten Partition muss bei GPT-Partitionierung jenseits der 32768 Byte liegen. Abbruch.",
		"mbr2tb" => "Eine MBR-Partitionierung von Festplatten mit mehr als zwei Terabyte ist nicht möglich. Abbruch.",
		"ntfschk" => "Prüfe NTFS auf Änderbarkeit...",
		"unplausible" => "Die bisherige Partitionierung scheint unplausibel. Abbruch.",
		"dryrun" => "Der Trockenlauf der NTFS-Größenänderung ist fehlgeschlagen. Bitte führen Sie auf dem Laufwerk eine Datenträgerprüfung durch und defragmentieren Sie es anschließend. Achten Sie auf wenigstens 20GB freien Platz. Abbruch.",
		"resize1" => "Verkleinere NTFS auf Partition 1...",
		"resizefailed" => "Die Durchführung der NTFS-Größenänderung ist fehlgeschlagen. Abbruch.",
		"resize2" => "Schreibe Partitionseintrag neu...",
		"resize3" => "Vergrößere NTFS auf Partition 1...",
		"waitpart" => "Warte auf Partitionen",
		"errorrestart" => "FEHLER - Neustart erforderlich!",
		"partnotfound" => "Installation fehlgeschlagen... Konnte erzeugte Partitionen nicht finden!",
		"part1" => "Setze Partitionseigenschaften...",
		"part2" => "Erstelle Dateisysteme...",
		"part3" => "Installation fehlgeschlagen... Konnte COMMAND nicht ausführen!",
		"part4" => "Installation fehlgeschlagen... Data-Partition konnte nicht ausgehängt werden!",
		"part5" => "Installiere Basissystem...",
		"part6" => "Installation fehlgeschlagen... System-Partition konnte nicht eingehängt werden!",
		"part7" => "Installiere Bootloader (BIOS)...",
		"part8" => "Installation fehlgeschlagen... System-Partition konnte nicht ausgehängt werden!",
		"part9" => "Installation fehlgeschlagen... UEFI-Partition konnte nicht eingehängt werden!",
		"part10" => "Installation fehlgeschlagen... UEFI-Partition konnte nicht ausgehängt werden!",
		"part11" => "Installation fehlgeschlagen... Signatur-Partition konnte nicht eingehängt werden!",
		"part12" => "Installation fehlgeschlagen... Signatur-Partition konnte nicht ausgehängt werden!",
		"wintitle" => "Desinfec't USB-Installation",
		"windesc" => "Dieses Werkzeug installiert desinfec't bootfähig auf einen wenigstens acht Gigabyte großen USB-Stick. Alle bislang auf dem USB-Stick gespeicherten Daten werden hierbei gelöscht. Die Installation dauert je nach Geschwindigkeit des USB-Sticks 5 bis 15 Minuten.",
		"betadesc" => "<b>BETA!</b> Dieses Werkzeug installiert desinfec't bootfähig auf eine wenigstens 32GB große USB-Festplatte (oder Stick), die bereits eine NTFS-Partition enthält. Die NTFS-Partition wird hierfür verkleinert, es wird versucht, vorhandene Daten zu erhalten. Die Installation dauert je nach Geschwindigkeit der USB-Festplatte 5 bis 15 Minuten.",
		"target" => "Ziellaufwerk:",
		"reread" => "Neu einlesen",
		"compat" => "Kompatibilität:",
		"compat2" => "Sowohl BIOS als auch UEFI unterstützen",
		"easyscan" => "Easy Scan:",
		"autostd" => "Automatik-Modus als Standard verwenden",
		"note" => "Achtung!",
		"betadesc2" => "Sie verwenden ein experimentelles Feature! Es besteht das Risiko, dass Daten auf der zu partitionierenden Festplatte verloren gehen. Führen Sie die Installation nur durch, wenn diese Daten mit vertretbarem Aufwand wieder hergestellt werden können.",
		"resizesync" => "Die Größenänderung des NTFS-Dateisystems wird synchron durchgeführt, in dieser Zeit reagiert das Programmfenster möglicherweise mehrere Minuten nicht. Brechen Sie das Programm keinesfalls ab!",
		"copysigs" => "Kopiere Signaturen",
		"filesystem" => "Dateisystem:",
		"use_btrfs_as_default" => "BTRFS als Standard nutzen (experimentell!)",
		"btrfs_requires_gpt" => "Die Verwendung von BTRFS erfordert die Auswahl des von UEFI-Systemen genutzten GPT-Partitionierungsschemas" ,
		"btrfs_needs_16g" => "Die Verwendung von BTRFS als Dateisystem erfordert einen USB-Stick mit wenigstens 16GB Kapazität.",
		"warn_btrfs" => "Achtung! Bei der BTRFS-Installation wird ein Partitionierungsschema verwendet, das unter Windows 10 dazu führt, dass bislang versteckte Laufwerke angezeigt werden und möglicherweise Laufwerksbuchstaben durcheinander gebracht werden. Verwenden Sie diese Option nur, wenn Sie genau wissen, was Sie tun!",
		"errorchecksum" => "Die Prüfung der System- oder Bootdateien ist fehlgeschlagen. Dies deutet auf einen defekten USB-Stick hin. Die Installation wird abgebrochen.",
		"errorchecksumtitle" => "Prüfsumme fehlerhaft",
		"checkfull" => "Überprüfung der Installation"
	},
	"nl" => {
		"nodrive" => "Geen geschikt medium gevonden",
		"reallywant" => "Weet je zeker dat je wilt installeren?",
		"reallydetails" => "Wil je Desinfec't echt op usb-stick MODEL VENDOR SIZE installeren?\n\n<b>Alle data op de stick worden daarbij gewist!</b>",
		"reallyresize" => "Wil je Desinfec't echt op usb-stick MODEL VENDOR SIZE installeren? Heb je het LEESMIJ-bestand bij dit programma gelezen?\n\n<b>Ondanks de andere NTFS-grootte kunnen alle data verloren gaan!</b>",
		"largedrive" => "<span size='large' weight='bold'>LET OP!</span>\n\nDe geselecteerde schijf VENDOR MODEL is\n\n<span size='large' weight='bold'>SIZE</span>\n\ngroot en moet compleet worden gewist. Weet je echt zeker dat het geen schijf is die je nog nodig hebt?\n\nKlik op \"Ja\" om de schijf te wissen en Desinfec't te installeren, met \"Nee\" keer je terug naar het menu met stations.",
		"progress" => "Installatievoortgang",
		"installing" => "De installatie wordt uitgevoerd, verwijder de usb-stick tijdens dit proces niet.",
		"installtitle" => "De installatie is voltooid",
		"installdone" => "De installatie is voltooid. Je kunt de usb-stick verwijderen.",
		"installdone2" => "De installatie is voltooid. De usb-stick wordt nu gebruikt en mag niet worden verwijderd. ",
		"delstick" => "USB-stick wissen...",
		"delmbr" => "MBR wissen...",
		"delgpt" => "Backup-GPT wissen...",
		"delpart" => "Partitie NUM wissen...",
		"mklabel" => "Partitietabel maken...",
		"mkpart" => "Partitie NUM maken...",
		"ptoolow" => "Het begin van de eerste partitie moet bij GPT-partitionering groter zijn dan 32768 bytes. Gestopt.",
		"mbr2tb" => "Bij harde schijven, waarvan de opslagcapaciteit groter is dan 2 TB, is geen MBR-partitionering mogelijk. Gestopt.",
		"ntfschk" => "NTFS controleren op wijzigingen...",
		"unplausible" => "De partitionering ljkt tot dusver niet plausibel. Gestopt.",
		"dryrun" => "De NTFS-grootte kon niet worden gewijzigd. Voer schijfcontrole op de harde schijf uit en defragmenteer de schijf vervolgens. Er moet minstens 20 GB aan vrije ruimte beschikbaar zijn. Gestopt. ",
		"resize1" => "NTFS op partitie 1 verkleinen...",
		"resizefailed" => "De NTFS-grootte kon niet worden gewijzigd. Gestopt.",
		"resize2" => "Partitie-entry opnieuw schrijven...",
		"resize3" => "NTFS op partitie 1 vergroten...",
		"waitpart" => "Wachten op partities",
		"errorrestart" => "FOUT – Start opnieuw op!",
		"partnotfound" => "Installatie mislukt... Kon gemaakte partities niet vinden!",
		"part1" => "Partitie-eigenschappen configureren...",
		"part2" => "Bestandssystemen maken...",
		"part3" => "Installatie mislukt... Kon COMMAND niet uitvoeren!",
		"part4" => "Installatie mislukt... Datapartitie kon niet worden geünmount!",
		"part5" => "Basissysteem installeren...",
		"part6" => "Installatie mislukt... Systeempartitie kon niet worden gemount!",
		"part7" => "Bootloader (BIOS) installeren...",
		"part8" => "Installatie mislukt... Systeempartitie kon niet worden geünmount!",
		"part9" => "Installatie mislukt... UEFI-partitie kon niet worden gemount!",
		"part10" => "Installatie mislukt... UEFI-partitie kon niet worden geünmount!",
		"part11" => "Installatie mislukt... Signaturepartitie kon niet worden gemount!",
		"part12" => "Installatie mislukt... Signaturepartitie kon niet worden geünmount!",
		"wintitle" => "Desinfec't USB-installatie",
		"windesc" => "Deze tool installeert desinfec't bootable op een usb-stick met minimaal acht gigabyte. Alle data die eerder op de stick stonden, worden daarbij gewist. De installatie neemt afhankelijk van de snelheid van de usb-stick 5 à 15 minuten in beslag.",
		"betadesc" => "<b>BETA!</b> Deze tool installeert desinfec't bootable op een usb-schijf (of -stick) met minimaal 32 GB waarop al een NTFS-partitie staat. De NTFS-partitie wordt hiervoor verkleind, er wordt geprobeerd de bestanden erop te behouden. De installatie neemt afhankelijk van de snelheid van de usb-schijf 5 à 15 minuten in beslag.",
		"target" => "Doelschijf:",
		"reread" => "Opnieuw inlezen",
		"compat" => "Compatibiliteit",
		"compat2" => "Zowel BIOS als UEFI ondersteunen",
		"easyscan" => "Easy Scan:",
		"autostd" => "Deze modus als standaard gebruiken",
		"note" => "Let op!",
		"betadesc2" => "Je gebruikt een experimentele feature! Er is een kans dat gegevens op de te partitioneren harde schijf verloren gaan. Voer de installatie alleen uit als je de gegevens kunt herstellen zonder (onredelijk) veel moeite te hoeven doen.",
		"resizesync" => "De grootte van het NTFS-bestandssysteem wordt synchroon aangepast; het programmavenster reageert in die tijd mogelijk meerdere minuten niet. Stop het programma in elk geval niet!",
		"copysigs" => "Copying virus signatures",
		"filesystem" => "File system:",
		"use_btrfs_as_default" => "Use BTRFS as default (considered BETA!)",
		"btrfs_requires_gpt" => "Installation on BTRFS requires activation of the GPT patrtitioning scheme used by UEFI."   ,
		"btrfs_needs_16g" => "For using BTRFS as default filesystem the target drive must have a size of at least 16GB.",
		"warn_btrfs" => "Achtung! Bei der BTRFS-Installation wird ein Partitionierungsschema verwendet, das unter Windows Creators Update dazu führt, dass bislang versteckte Laufwerke angezeigt werden und möglicherweise Laufwerksbuchstaben durcheinander gebracht werden. Verwenden Sie diese Option nur, wenn Sie genau wissen, was Sie tun!"
	} 
}

def get_trans(id) 
	lang = ENV["LANG"][0..1] 
	return id unless LOCSTRINGS.has_key? lang
	return id unless LOCSTRINGS[lang].has_key? id
	return LOCSTRINGS[lang][id] 
end

#### Global variables

@loopdevs = []
@opts = OptionParser.new 
@keepntfs = false
@alldisks = false 
@btrfssize = 0
@thisyear = 202021
@opts.on("--keep-ntfs")  {  @keepntfs = true }
@opts.on("--all-disks")  {  @alldisks = true }
@opts.on("--loopdevs", :REQUIRED) { |i| @loopdevs = i.split(",") }
@opts.on("--btrfs-size", :REQUIRED) { |i| @btrfssize = i.to_i }
@opts.parse!
@drivelist = Array.new
@sysrepvte = Vte::Terminal.new 

#### Commonly used functions

def sys_rep(command, vte, pgbar=nil) 
	vte.reset(true, true)
	vte_running = true
	vte.signal_connect("child_exited") { vte_running = false }
	current_pid = vte.fork_command(command[0], command)
	puts current_pid.to_s 
	while (vte_running == true) 
		pgbar.pulse unless pgbar.nil?
		while (Gtk.events_pending?)
			Gtk.main_iteration
		end
		sleep 0.1
	end
end

def iopo_rep(command, vte=nil, pgbar=nil)
	if vte.nil?
		# return system(command)
		vte = @sysrepvte
	end
	vte.reset(true, true)
	vte_running = true
	vte.signal_connect("child_exited") { vte_running = false }
	outlines = []
	cmdfile = "/tmp/script_" + rand(1_000_000_000).to_s
	outfile = "/tmp/output_" + rand(1_000_000_000).to_s
	s = File.new(cmdfile, "w")
	s.write(command + " > " + outfile + "\n")
	s.close
	current_pid = vte.fork_command("bash", [ "bash", cmdfile] )
	puts current_pid.to_s 
	while (vte_running == true) 
		pgbar.pulse unless pgbar.nil?
		while (Gtk.events_pending?)
			Gtk.main_iteration
		end
		sleep 0.1
	end
	File.unlink(cmdfile)
	File.open(outfile).each { |line|
		outlines.push line.strip
	}
	File.unlink(outfile)
	return outlines
end

def sys_rep_res(command, vte=nil, pgbar=nil)
	if vte.nil?
		# return system(command)
		vte = @sysrepvte
	end
	vte.reset(true, true)
	vte_running = true
	vte.signal_connect("child_exited") { vte_running = false }
	outlines = []
	cmdfile = "/tmp/script_" + rand(1_000_000_000).to_s
	statfile = "/tmp/stat_" + rand(1_000_000_000).to_s
	s = File.new(cmdfile, "w")
	s.write(command + "\n")
	s.write("echo $? > " + statfile + "\n")
	s.close
	current_pid = vte.fork_command("bash", [ "bash", cmdfile] )
	puts current_pid.to_s 
	while (vte_running == true) 
		pgbar.pulse unless pgbar.nil?
		while (Gtk.events_pending?)
			Gtk.main_iteration
		end
		sleep 0.1
	end
	File.unlink(cmdfile)
	retval = 1
	File.open(statfile).each { |line|
		retval = line.strip.to_i if line.strip.size > 0
	}
	File.unlink(statfile)
	return true if retval < 1
	return false 
end

def error_dialog(title, text) 
	dialog = Gtk::Dialog.new(
		title,
		$mainwindow,
		Gtk::Dialog::MODAL,
		[ Gtk::Stock::OK, Gtk::Dialog::RESPONSE_OK ]
	)
	dialog.default_response = Gtk::Dialog::RESPONSE_OK
	dialog.has_separator = false
	label = Gtk::Label.new
	label.set_markup(text)
	label.wrap = true
	image = Gtk::Image.new(Gtk::Stock::DIALOG_ERROR, Gtk::IconSize::DIALOG)
	hbox = Gtk::HBox.new(false, 5)
	hbox.border_width = 10
	hbox.pack_start_defaults(image)
	hbox.pack_start_defaults(label)
	dialog.vbox.add(hbox)
	dialog.show_all
	dialog.run { |resp|
		dialog.destroy
		return true
	}
end

def umount_stick(device)
	device.partitions.each { |p| 
		p.umount
	}
	return device.mounted 
end

def fill_drive_combo(dropdown, apply)
	99.downto(0) { |n|
		begin
			dropdown.remove_text(n)
		rescue
		end
	}
	@drivelist = Array.new
	drives = Array.new
	Dir.entries("/sys/block").each { |l|
		drives.push(MfsDiskDrive.new(l, true)) if ( l =~ /[a-z]$/ || @loopdevs.include?(l) )  
	}
	drives.each { |d| 
		if d.usb == true && d.size > 7_500_000_000 && ( d.mounted == false || umount_stick(d) == false )
			if @keepntfs == false
				@drivelist.push d
				dropdown.append_text "#{d.vendor} #{d.model} #{d.device} #{d.human_size}"
			elsif d.partitions[0].fs =~ /ntfs/i && d.partitions.size < 2 && d.size > 55_000_000_000	
				@drivelist.push d
				dropdown.append_text "#{d.vendor} #{d.model} #{d.device} #{d.human_size}"
			end
		elsif @alldisks == true && d.size > 11_500_000_000 && ( d.mounted == false || umount_stick(d) == false )
			@drivelist.push d
			dropdown.append_text "#{d.vendor} #{d.model} #{d.device} #{d.human_size}"
		end
	} 
	if @drivelist.size > 0
		apply.sensitive = true
		dropdown.sensitive = true
	else
		apply.sensitive = false
		dropdown.sensitive = false
		dropdown.append_text get_trans("nodrive") 
	end
	dropdown.active = 0
	
end

#### Modal dialog to confirm

def confirm_dialog(drive)
	dialog = Gtk::Dialog.new(
		get_trans("reallywant"),
		$mainwindow,
		Gtk::Dialog::MODAL,
		[ Gtk::Stock::YES, Gtk::Dialog::RESPONSE_YES ],
		[ Gtk::Stock::NO, Gtk::Dialog::RESPONSE_NO ]
	)
	dialog.default_response = Gtk::Dialog::RESPONSE_NO
	dialog.has_separator = false
	label = Gtk::Label.new
	label.set_markup(get_trans("reallydetails").gsub("MODEL", drive.model).gsub("VENDOR", drive.vendor).gsub("SIZE", drive.human_size))
	label.set_markup(get_trans("reallyresize").gsub("MODEL", drive.model).gsub("VENDOR", drive.vendor).gsub("SIZE", drive.human_size)) if @keepntfs == true 
	label.wrap = true
	image = Gtk::Image.new(Gtk::Stock::DIALOG_WARNING, Gtk::IconSize::DIALOG)
	hbox = Gtk::HBox.new(false, 5)
	hbox.border_width = 10
	hbox.pack_start_defaults(image)
	hbox.pack_start_defaults(label)
	dialog.vbox.add(hbox)
	dialog.show_all
	dialog.run { |resp|
		case resp
		when Gtk::Dialog::RESPONSE_YES
			dialog.destroy
			return true
		else
			dialog.destroy
			return false
		end
	}
end

def confirm_again_dialog(drive)
	dialog = Gtk::Dialog.new(
		get_trans("reallywant"),
		$mainwindow,
		Gtk::Dialog::MODAL,
		[ Gtk::Stock::YES, Gtk::Dialog::RESPONSE_YES ],
		[ Gtk::Stock::NO, Gtk::Dialog::RESPONSE_NO ]
	)
	dialog.default_response = Gtk::Dialog::RESPONSE_NO
	dialog.has_separator = false
	label = Gtk::Label.new
	label.set_markup(get_trans("largedrive").gsub("MODEL", drive.model).gsub("VENDOR", drive.vendor).gsub("SIZE", drive.human_size))
	label.wrap = true
	image = Gtk::Image.new(Gtk::Stock::DIALOG_WARNING, Gtk::IconSize::DIALOG)
	hbox = Gtk::HBox.new(false, 5)
	hbox.border_width = 10
	# hbox.pack_start_defaults(image)
	hbox.pack_start_defaults(label)
	dialog.vbox.add(hbox)
	dialog.show_all
	dialog.run { |resp|
		case resp
		when Gtk::Dialog::RESPONSE_YES
			dialog.destroy
			return true
		else
			dialog.destroy
			return false
		end
	}
end

#### Update progress bar

def update_pgbar(pgbar, text) 
	while (Gtk.events_pending?)
		Gtk.main_iteration
	end
	pgbar.text = text
	pgbar.pulse
	while (Gtk.events_pending?)
		Gtk.main_iteration
	end
end

#### Modal dialog for installation progress

def install_progress(drive, gptprefered=true, easymode=false, use_btrfs=false)
	dialog = Gtk::Dialog.new(
		get_trans("progress"),
		$mainwindow,
		Gtk::Dialog::MODAL
	)
	dialog.deletable = false
	label = Gtk::Label.new
	label.width_request = 400
	label.set_markup(get_trans("installing"))
	label.wrap = true
	image = Gtk::Image.new(Gtk::Stock::DIALOG_INFO, Gtk::IconSize::DIALOG)
	hbox = Gtk::HBox.new(false, 5)
	hbox.border_width = 10
	hbox.pack_start_defaults(image)
	hbox.pack_start_defaults(label)
	pgbar = Gtk::ProgressBar.new
	okbutton = Gtk::Button.new(Gtk::Stock::OK)
	okbutton.sensitive = false
	dialog.vbox.add(hbox)
	dialog.vbox.add(pgbar)
	dialog.add_action_widget(okbutton, Gtk::Dialog::RESPONSE_OK)
	dialog.show_all
	if use_btrfs == false
		unpluggable = run_hacked_installation(drive, pgbar, gptprefered, easymode, use_btrfs)
	else
		unpluggable = run_installation(drive, pgbar, gptprefered, easymode, use_btrfs)
	end
	if unpluggable == true
		pgbar.text = get_trans("installtitle")
		label.set_markup(get_trans("installdone"))
	else
		pgbar.text = get_trans("installtitle")
		label.set_markup(get_trans("installdone2"))
	end
	pgbar.fraction = 1.0 
	okbutton.sensitive = true
	dialog.run
	dialog.destroy
end

#### Run the real installation 

def run_hacked_installation(target, pgbar, gptprefered=true, easymode=false, use_btrfs=false)
	# some calculations
	sigsize = 2048
	swapsize = 256
	systemsize = 2816
	
	systeminfo = ` df -k /cdrom | tail -n1 `.strip.split 
	systemsize = systeminfo[2].to_i / 1024
	
	# FIXME: calculate sys_rep_res size
	if target.size > 30_000_000_000
		swapsize = 3072
		sigsize = 8192
		systemsize += 512
	elsif target.size > 15_000_000_000
		sigsize = 6144
		swapsize = 1024
		systemsize += 256
	elsif target.size > 7_000_000_000
		sigsize = 4096
		swapsize = 1024
		systemsize += 64
	end
	uuid = nil
	puts target.size.to_s
	kblocks = target.size / 1024
	puts kblocks.to_s
	
	# FIXME: add strings
	# First install the ISO to the target
	# Calculate number of 8M Blocks (plus one...)
	emblocks = systemsize / 8 
	pgbar.text = get_trans("part5")
	0.upto(emblocks) { |n|
		sys_rep_res("dd if=#{systeminfo[0]} of=/dev/#{target.device} bs=8388608 count=1 seek=#{n} skip=#{n} ; sync ") 
		# sys_rep_res("sync")
		pgbar.fraction = n.to_f / emblocks.to_f 
		while (Gtk.events_pending?)
			Gtk.main_iteration
		end
	}
	
	# zap MBR
	sys_rep_res("dd if=/dev/zero bs=1024 count=32 of=/dev/#{target.device}")
	# Create partition table and single FAT32 partition 
	command = "parted -s /dev/#{target.device} unit B mklabel msdos"
	puts command
	sys_rep_res command
	systembytes = systemsize * 1024 * 1024 
	partitionend = ( kblocks / 1024 - sigsize ) * 1024 * 1024 - 1   
	update_pgbar(pgbar, get_trans("mkpart").gsub("NUM", "1") )
	command = "parted -s /dev/#{target.device} unit B mkpart primary fat32 #{systembytes.to_s} #{partitionend.to_s} "
	puts command 
	sys_rep_res command
	flags = [ "boot" ]
	flags.push("esp") if gptprefered == true
	flags.each { |t|
		sys_rep_res("parted -s /dev/#{target.device} set 1 #{t} on")
	}
	0.upto(500) { |n|
		unless File.blockdev?("/dev/#{target.device}1")
			update_pgbar(pgbar, get_trans("waitpart") )
			sleep 0.2
		end
	}
	unless File.blockdev?("/dev/#{target.device}1")
		# puts "Installation fehlgeschlagen... Konnte erzeugte Partitionen nicht finden!"  
		error_dialog(get_trans("errorrestart"), get_trans("partnotfound"))
		exit 1
	end
	sys_rep_res("dd if=/dev/zero bs=1M count=8 of=/dev/#{target.device}1") 
	command = "mkfs.msdos -F32 -n desinfDATA /dev/#{target.device}1"
	puts command 
	sys_rep_res command
	# Install legacy MBR
	command = "dd if=/usr/lib/SYSLINUX/mbr.bin bs=440 count=1 of=/dev/#{target.device}"
	puts command
	sys_rep_res command
	# Install bootfiles
	sys_rep_res("mkdir -p /desinfect/target")
	sys_rep_res("mount /dev/#{target.device}1 /desinfect/target")
	unless sys_rep_res("mountpoint /desinfect/target")
		# puts "Installation fehlgeschlagen... Konnte erzeugte Partitionen nicht finden!"  
		error_dialog(get_trans("errorrestart"), get_trans("partnotfound"))
		exit 1
	end
	# sys_rep_res("tar -C /cdrom --exclude=filesystem.squashfs -cf - boot isolinux EFI casper autorun.inf ct.ico 'Desinfect Hilfe' .disk LIESMICH.txt preseed README.diskdefines shutdown.bat 'TeamViewer Portable' | tar -C /desinfect/target -xf - ")
	siginst_running = true
	sigs_term = Vte::Terminal.new
	sigs_term.fork_command("bash", [ "bash", "copy1.sh"] )
	sigs_term.signal_connect("child-exited") { siginst_running = false } 
	update_pgbar(pgbar, get_trans("part7") )
	while siginst_running == true 
		sleep 0.5
		pgbar.pulse
		while (Gtk.events_pending?)
			Gtk.main_iteration
		end
	end
	FileUtils.cp("/desinfect/target/isolinux/isolinux.cfg", "/desinfect/target/isolinux/syslinux.cfg")
	sys_rep_res("sed -i 's%#menu hide%menu hide%g' /desinfect/target/isolinux/os.cfg; sleep 3")
	sys_rep_res("sed -i 's%include local.cfg%#include local.cfg%g' /desinfect/target/isolinux/syslinux.cfg; sleep 3")
	FileUtils.cp("/desinfect/target/boot/grub/grub.cfg.usb", "/desinfect/target/boot/grub/grub.cfg")
	if easymode == true
		sys_rep_res('sed -i \'s/label desinfect-easy/label desinfect-easy\n  menu default/g\' /desinfect/target/isolinux/os.cfg')  
		sys_rep_res('sed -i \'s/set default=0/set default=1/g\' /desinfect/target/boot/grub/grub.cfg') 
	end
	sys_rep_res('sed -i \'s/quiet/quiet noprompt noeject/g\' /desinfect/target/isolinux/os.cfg')  
	sys_rep_res('sed -i \'s/quiet/quiet noprompt noeject/g\' /desinfect/target/boot/grub/grub.cfg')  
	sys_rep_res("sed -i 's%^shellexecute%;shellexecute%g' /desinfect/target/autorun.inf") 
	sys_rep_res("/opt/desinfect/usb-installer/extlinux -i /desinfect/target/isolinux") 
	FileUtils.mv("/desinfect/target/EFI", "/desinfect/target/unused.EFI") unless gptprefered == true
	sys_rep_res("sync; sleep 3")
	sys_rep_res("umount /desinfect/target") 
	# Mount the full filesys_rep_res and run checksum
	sys_rep_res("mount -t iso9660 /dev/#{target.device} /desinfect/target")
	siginst_running = true
	sigs_term.fork_command("bash", [ "bash", "check.sh"] )
	update_pgbar(pgbar, get_trans("checkfull") )
	while siginst_running == true 
		sleep 0.5
		pgbar.pulse
		while (Gtk.events_pending?)
			Gtk.main_iteration
		end
	end
	sys_rep_res("umount /desinfect/target") 
	# Now compare checksums
	check_main = 1
	check_boot = 1
	begin
		check_main = File.read("/tmp/check_main.txt").strip.to_i
		check_boot = File.read("/tmp/check_boot.txt").strip.to_i
	rescue
		# dafaq?
	end
	if (check_main + check_boot > 0)
		error_dialog(get_trans("errorchecksum"), get_trans("errorchecksumtitle"))
		exit 1
	end

	# Find the beginning of the signatures "partition"
	emblocks = partitionend / 1024 / 8192 + 1 
	sigstart = emblocks * 1024 * 8192
	freeloop = iopo_rep("losetup -f")[0].strip  
	command = "losetup -o #{sigstart.to_s} #{freeloop} /dev/#{target.device}"
	puts command
	sys_rep_res command
	command = "dd if=/dev/zero bs=1M count=8 of=#{freeloop}"
	puts command
	sys_rep_res command
	sys_rep_res("mkfs.ext4 -L desinfSIGS #{freeloop}")
	sys_rep_res("mount #{freeloop} /desinfect/target") 
	unless sys_rep_res("mountpoint /desinfect/target")
		# puts "Installation fehlgeschlagen... Konnte erzeugte Partitionen nicht finden!"  
		error_dialog(get_trans("errorrestart"), get_trans("partnotfound"))
		exit 1
	end
	sys_rep_res("mkdir -p /desinfect/target/desinfect-signatures")
	sys_rep_res("mkdir -p /desinfect/target/deb")
	[ "kaspersky", "eset", "avira", "clamav", "sophos-av", "f-secure" ].each { |d|
		sys_rep_res("mkdir -p /desinfect/target/desinfect-signatures/#{d}")
	}
	# Let our script do the real copy job
	siginst_running = true
	sigs_term.fork_command("bash", [ "bash", "copy-sigs.sh"] )
	update_pgbar(pgbar, get_trans("copysigs") )
	while siginst_running == true 
		sleep 0.5
		pgbar.pulse
		while (Gtk.events_pending?)
			Gtk.main_iteration
		end
	end
	# Create swap file
	sys_rep_res("dd if=/dev/zero bs=1M count=1 seek=#{swapsize.to_s} of=/desinfect/target/swap.img")
	sys_rep_res("mkswap /desinfect/target/swap.img") 
	# Unmount signatures
	sys_rep_res("sync") 
	unless sys_rep_res("umount #{freeloop}") 
		sleep 2
		sys_rep_res("umount #{freeloop}") 
	end
	if sys_rep_res("mountpoint /desinfect/target")
		# puts "Installation fehlgeschlagen... Signatur-Partition konnte nicht ausgehängt werden!"  
		error_dialog(get_trans("errorrestart"), get_trans("part12") )
		exit 1
	end
	# ...or unmount
	retval = false
	
	if sys_rep_res("mountpoint /opt/desinfect/signatures")
		# There already is a stick present, tell the user he can remove the new one
		1.upto(5) { |n| sys_rep_res("umount /dev/#{target.device}#{n}") } 
		retval = true
		sys_rep_res("losetup -d #{freeloop}")
	else
		# FIXME: Create loop device for desinfDATA
		# There is no stick present, use our stick
		sys_rep_res("mount #{freeloop} /opt/desinfect/signatures")
		sys_rep_res("umount /var/kl/bases_rd")
		# FIXME! Umount some some potentially mounted tmpfs directories
		sys_rep_res("/etc/init.d/esets stop")
		sys_rep_res("/etc/init.d/fsaua stop")
		{ "avira" => "/AntiVir", 
		  "clamav" =>  "/var/lib/clamav", 
		  "kaspersky" => "/var/kl/bases_rd", 
		  "eset" => "/var/opt/eset",
		   "sophos-av" => "/opt/sophos-av",
		   "f-secure" => "/var/opt/f-secure" 
		}.each { |k,v| 
			sys_rep_res("mount --bind /opt/desinfect/signatures/desinfect-signatures/#{k} #{v}")
		}
		sys_rep_res("/etc/init.d/esets start")
		FileUtils.mkdir_p("/tmp/desinfect_logs")
		FileUtils.mkdir_p("/opt/desinfect/signatures/desinfect_logs")
		sys_rep_res("touch /opt/desinfect/signatures/.desinfect" + @thisyear.to_s + " ; sleep 2") 
		sys_rep_res("sync") 
		sys_rep_res("mount --bind /opt/desinfect/signatures/desinfect_logs /tmp/desinfect_logs")
		swaploop = ` losetup -f `.strip
		sys_rep_res("losetup #{swaploop} /opt/desinfect/signatures/swap.img")
		sys_rep_res("swapon #{swaploop}")
		retval = false
	end
	return retval
end

def run_installation(target, pgbar, gptprefered=true, easymode=false, use_btrfs=false)
	# some calculations
	sigsize = 1280
	swapsize = 256
	btrfssize = 4096
	btrfssize = @btrfssize if @btrfssize > 0
	systemsize = 2816
	if target.size > 30_000_000_000
		btrfssize = 12288 if @btrfssize < 1
		swapsize = 3072
		sigsize = 8192
	elsif target.size > 15_000_000_000
		sigsize = 6144
		btrfssize = 6144 if @btrfssize < 1
		swapsize = 1024
	elsif target.size > 7_000_000_000
		sigsize = 4096
		swapsize = 1024
	end
	uuid = nil
	puts target.size.to_s
	kblocks = target.size / 1024
	puts kblocks.to_s
	
	# Create partitions
	pend = Array.new
	if use_btrfs && gptprefered == true
		systemsize = 256
		pend[4] = target.size - 1048577
		pend[3] = pend[4] - ( btrfssize * 1048576)
	else
		# Fixed size for partition 4 - UEFI boot
		pend[3] = target.size - 1048577
	end
	
	# Fixed size for partition 3 - signatures, swapfile, persistent data
	pend[2] = pend[3] - ( 256 * 1048576)
	pend[2] = pend[3] - ( 4 * 1048576) if gptprefered == false 
	# Fixed size for partition 2 - sys_rep_res and BIOS boot
	pend[1] = pend[2] - ( sigsize * 1048576)
	# Flexible size for partition 1 - user data
	pend[0] = pend[1] - ( systemsize * 1048576)
	
	if (@keepntfs == false)
		update_pgbar(pgbar, get_trans("delstick"))
		# Zap partition contents
		5.downto(1) { |n|
			if sys_rep_res("test -b /dev/#{target.device}#{n.to_s}")
				update_pgbar(pgbar, get_trans("delpart").gsub("NUM", n.to_s ) )
				command = "dd if=/dev/zero bs=1024 count=1024 of=/dev/#{target.device}#{n.to_s} oflag=sync"
				puts command
				sys_rep_res command
			end
		}
		# Zap GPT / MBR
		update_pgbar(pgbar, get_trans("delmbr"))
		command = "dd if=/dev/zero bs=1024 count=1024 of=/dev/#{target.device} oflag=sync"
		puts command
		sys_rep_res command
		update_pgbar(pgbar, get_trans("delgpt") )
		command = "dd if=/dev/zero bs=1024 count=2048 of=/dev/#{target.device} seek=#{kblocks - 2048} oflag=sync"
		puts command 
		sys_rep_res command
		update_pgbar(pgbar, get_trans("mklabel") )
		if gptprefered == true
			command = "parted -s /dev/#{target.device} unit B mklabel gpt"
		else
			command = "parted -s /dev/#{target.device} unit B mklabel msdos"
		end
		puts command
		sys_rep_res command
		update_pgbar(pgbar, get_trans("mkpart").gsub("NUM", "1") )
		command = "parted -s /dev/#{target.device} unit B mkpart primary fat32 1048576 #{pend[0].to_s} "
		puts command 
		sys_rep_res command
	else
		partstart = 0
		partend = 0
		# Check for start at block 63 or beyond
		IO.popen("parted -s -m /dev/#{target.device} unit b print") { |line|
			while line.gets 
				if $_ =~ /^[0-9]/ 
					ltoks = $_.strip.split(":")
					partstart = ltoks[1].gsub("B", "").to_i
					partend = ltoks[2].gsub("B", "").to_i
				end
			end
		}
		# Check start of partition
		if partstart < 32768 && gptprefered == true
			error_dialog("Error", get_trans("ptoolow") )
			exit 1
		end
		if target.size > 2199023255552 && gptprefered == false
			error_dialog("Error", get_trans("mbr2tb"))
			exit 1
		end
		update_pgbar(pgbar, get_trans("ntfschk"))
		# Assume 16GB for Desinfec't
		psize = 17179869184 - partstart 
		if psize < 8589934592
			error_dialog("Error", get_trans("unplausible"))
			exit 1 
		end
		update_pgbar(pgbar, get_trans("ntfschk"))
		# Check for resizeability
		unless sys_rep_res("echo y | ntfsresize --force -s #{psize} -n /dev/#{target.partitions[0].device}") 
			error_dialog("Error", get_trans("dryrun") )
			exit 1
		end
		update_pgbar(pgbar, get_trans("resize1")  )
		# resize NTFS
		unless sys_rep_res("echo y | ntfsresize --force -s #{psize} /dev/#{target.partitions[0].device}") 
			error_dialog("Error", get_trans("resizefailed") )
			exit 1
		end
		update_pgbar(pgbar, get_trans("mklabel"))
		# delete partition table, recreate partition table as GPT
		if gptprefered == true
			sys_rep_res("parted -s /dev/#{target.device} mklabel gpt") 
		else
			sys_rep_res("parted -s /dev/#{target.device} mklabel mbr")
		end		
		update_pgbar(pgbar, get_trans("mkpart"))
		# recreate partition
		sys_rep_res("parted -s /dev/#{target.device} unit B mkpart primary ntfs #{partstart} #{pend[0].to_s} ")
		# stretch NTFS again
		update_pgbar(pgbar, get_trans("resize3") )
		sys_rep_res("echo y | ntfsresize --force /dev/#{target.partitions[0].device}") 
	end
	
	update_pgbar(pgbar, get_trans("mkpart").gsub("NUM", "2" )) 
	command = "parted -s /dev/#{target.device} unit B mkpart primary fat32 #{ (pend[0] + 1).to_s} #{pend[1].to_s} "
	puts command
	sys_rep_res command
	update_pgbar(pgbar, get_trans("mkpart").gsub("NUM", "3" ))
	command = "parted -s /dev/#{target.device} unit B mkpart primary ext2 #{ (pend[1] + 1).to_s} #{pend[2].to_s} "
	puts command 
	sys_rep_res command
	update_pgbar(pgbar,get_trans("mkpart").gsub("NUM", "4" ))
	command = "parted -s /dev/#{target.device} unit B mkpart primary fat32 #{ (pend[2] + 1).to_s} #{pend[3].to_s} "
	puts command
	sys_rep_res command
	if gptprefered == true && use_btrfs == true
		command = "parted -s /dev/#{target.device} unit B mkpart primary ext2 #{ (pend[3] + 1).to_s} #{pend[4].to_s} "
		puts command
		sys_rep_res command
	end
	sys_rep_res "sync" 
	sys_rep_res("kpartx -a /dev/#{target.device}") if ( target.device =~ /^loop/ )
	# sleep 5
	0.upto(500) { |n|
		unless sys_rep_res("test -b /dev/#{target.device}4") || sys_rep_res("test -b /dev/mapper/#{target.device}p4")
			update_pgbar(pgbar, get_trans("waitpart") )
			sleep 0.2
		end
	}
	unless sys_rep_res("test -b /dev/#{target.device}4") || sys_rep_res("test -b /dev/mapper/#{target.device}p4")
		# puts "Installation fehlgeschlagen... Konnte erzeugte Partitionen nicht finden!"  
		error_dialog(get_trans("errorrestart"), get_trans("partnotfound"))
		exit 1
	end
	0.upto(15) { |n|
		sleep 0.2
		update_pgbar(pgbar, get_trans("waitpart"))
	}
	
	# Mount desinfDATA - is needed later!
	unless sys_rep_res("mountpoint -q /media/desinfDATA")
		if sys_rep_res("test -b /dev/disk/by-label/desinfDATA")
			sys_rep_res("mkdir -p /media/desinfDATA")
			sys_rep_res("mount -t vfat -o rw,uid=999,gid=999 /dev/disk/by-label/desinfDATA /media/desinfDATA") 
		end
	end
	
	if File.directory?("/cdrom/casper/filesystem.dir") 
		unless sys_rep_res("mountpoint -q /media/desinfEFI")
			if sys_rep_res("test -b /dev/disk/by-label/desinfEFI")
				sys_rep_res("mkdir -p /media/desinfEFI")
				sys_rep_res("mount -t vfat -o ro,uid=999,gid=999 /dev/disk/by-label/desinfEFI /media/desinfEFI") 
			end
		end
		unless sys_rep_res("mountpoint -q /media/desinfSYS")
			if sys_rep_res("test -b /dev/disk/by-label/desinfSYS")
				sys_rep_res("mkdir -p /media/desinfSYS")
				sys_rep_res("mount -t vfat -o ro,uid=999,gid=999 /dev/disk/by-label/desinfSYS /media/desinfSYS") 
			end
		end
	end
	
	
	1.upto(5) { |n| 
		sys_rep_res("umount -f /dev/#{target.device}#{n}")
		sys_rep_res("umount -f /dev/mapper/#{target.device}p#{n}")
	}
	update_pgbar(pgbar, get_trans("part1"))
	if gptprefered == true
		# Set boot flag UEFI
		# Set legacy boot flag BIOS
		command = "parted -s /dev/#{target.device} unit B set 4 boot on"
		puts command
		sys_rep_res command
		command = "parted -s /dev/#{target.device} unit B set 2 legacy_boot on"
		puts command
		sys_rep_res command
		# Install legacy MBR
		command = "dd if=/usr/lib/SYSLINUX/gptmbr.bin bs=440 count=1 of=/dev/#{target.device}"
		puts command
		sys_rep_res command
	else
		command = "parted -s /dev/#{target.device} unit B set 2 boot on"
		puts command
		sys_rep_res command
		# Install legacy MBR
		command = "dd if=/usr/lib/SYSLINUX/mbr.bin bs=440 count=1 of=/dev/#{target.device}"
		puts command
		sys_rep_res command
	end
	sys_rep_res("sync")
	# Make filesystems
	fscomm = []
	if target.device =~ /^loop/ 
		fscomm = [ "dd if=/dev/zero bs=1M count=2 of=/dev/mapper/#{target.device}p2", 
			"mkfs.msdos -F32 -n desinfSYS /dev/mapper/#{target.device}p2", 
			"dd if=/dev/zero bs=1M count=2 of=/dev/mapper/#{target.device}p3", 
			"mkfs.ext4 -L desinfSIGS /dev/mapper/#{target.device}p3" ]
		fscomm.push("dd if=/dev/zero bs=1M count=2 of=/dev/mapper/#{target.device}p1") if @keepntfs == false 
		fscomm.push("mkfs.msdos -F32 -n desinfDATA /dev/mapper/#{target.device}p1") if @keepntfs == false 
		fscomm.push("dd if=/dev/zero bs=1M count=2 of=/dev/mapper/#{target.device}p4") if  gptprefered == true
		fscomm.push("mkfs.msdos -F32 -n desinfEFI /dev/mapper/#{target.device}p4") if gptprefered == true
		fscomm.push("dd if=/dev/zero bs=1M count=2 of=/dev/mapper/#{target.device}p5") if gptprefered == true && use_btrfs
		fscomm.push("mkfs.btrfs -L desinfBTRFS -f /dev/mapper/#{target.device}p5") if gptprefered == true && use_btrfs
	else
		fscomm = [  "dd if=/dev/zero bs=1M count=2 of=/dev/#{target.device}2",
			"mkfs.msdos -F32 -n desinfSYS /dev/#{target.device}2", 
			"dd if=/dev/zero bs=1M count=2 of=/dev/#{target.device}3",
			"mkfs.ext4 -L desinfSIGS /dev/#{target.device}3" ]
		fscomm.push("dd if=/dev/zero bs=1M count=2 of=/dev/#{target.device}1") if @keepntfs == false 
		fscomm.push("mkfs.msdos -F32 -n desinfDATA /dev/#{target.device}1") if @keepntfs == false 
		fscomm.push("dd if=/dev/zero bs=1M count=2 of=/dev/#{target.device}4") if gptprefered == true
		fscomm.push("mkfs.msdos -F32 -n desinfEFI /dev/#{target.device}4") if gptprefered == true
		fscomm.push("dd if=/dev/zero bs=1M count=2 of=/dev/#{target.device}5") if gptprefered == true && use_btrfs
		fscomm.push("mkfs.btrfs -L desinfBTRFS -f /dev/#{target.device}5") if gptprefered == true && use_btrfs
	end
	fscomm.each { |m|
		update_pgbar(pgbar, get_trans("part2"))
		puts m
		unless sys_rep_res(m)
			sleep 2
			unless sys_rep_res(m)
				sleep 5
				unless sys_rep_res(m)
					# puts "Installation fehlgeschlagen... Konnte #{m} nicht ausführen!"  
					error_dialog(get_trans("error_restart"), get_trans("part3").gsub("COMMAND", m)  )
					exit 1
				end
			end
		end
		sys_rep_res("sync") 
	}
	sleep 5
	1.upto(4) { |n| 
		sys_rep_res("umount -f /dev/#{target.device}#{n}")
		sys_rep_res("umount -f /dev/mapper/#{target.device}p#{n}")
	} 
	# Retrieve UUID for GRUB 
	blkidcomm = "blkid -o udev /dev/#{target.device}2"
	blkidcomm = "blkid -o udev /dev/mapper/#{target.device}p2" if target.device =~ /^loop/
	iopo_rep(blkidcomm).each { |l|
		toks = l.strip.split("=")
		uuid = toks[1] if toks[0] == "ID_FS_UUID"
	}
	
	# Install shutdown.bat
	sys_rep_res("mkdir -p /desinfect/target")
	sys_rep_res("mount -t vfat /dev/#{target.device}1 /desinfect/target") || sys_rep_res("mount -t ntfs-3g /dev/#{target.device}1 /desinfect/target") || sys_rep_res("mount -t vfat /dev/mapper/#{target.device}p1 /desinfect/target") || sys_rep_res("mount -t ntfs-3g /dev/mapper/#{target.device}p1 /desinfect/target")
	if File.exists?("/cdrom/shutdown.bat")
		sys_rep_res("cp -v /cdrom/shutdown.bat /desinfect/target")
	else
		sys_rep_res("cp -v /opt/desinfect/usb-installer/shutdown.bat /desinfect/target")
	end
	tvdir = nil
	if File.directory?("/cdrom/TeamViewer Portable")
		tvdir = "/cdrom/"
	elsif File.directory?("/isodevice/TeamViewer Portable")
		tvdir = "/isodevice/"
	elsif File.directory?("/media/desinfDATA/TeamViewer Portable")
		tvdir = "/media/desinfDATA/"	
	end
	sys_rep_res("cp -vr #{tvdir}'TeamViewer Portable' /desinfect/target/" ) unless tvdir.nil?
	win8dir = nil
	if File.directory?("/cdrom/Windows 8 Hilfe")
		win8dir = "/cdrom/"
	elsif File.directory?("/isodevice/Windows 8 Hilfe")
		win8dir = "/isodevice/"
	elsif File.directory?("/media/desinfDATA/Windows 8 Hilfe")
		win8dir = "/media/desinfDATA/"	
	end
	sys_rep_res("cp -vr #{win8dir}'Windows 8 Hilfe' /desinfect/target/" ) unless win8dir.nil?
	sys_rep_res("cp -vr /cdrom/'Desinfect Hilfe' /desinfect/target/" )
	sys_rep_res("cp -v /cdrom/hiberfil-abschalten.bat /desinfect/target")
	sys_rep_res("cp -v /cdrom/hiberfil-anschalten.bat /desinfect/target")
	sys_rep_res("cp -v /cdrom/shutdown.bat /desinfect/target")
	sys_rep_res("cp -v /cdrom/autorun.inf /desinfect/target")
	sys_rep_res("cp -v /cdrom/ct.ico /desinfect/target")
	sys_rep_res("sed -i 's%^shellexecute%;shellexecute%g' /desinfect/target/autorun.inf")
	sys_rep_res("sync")
	unless sys_rep_res("umount /dev/#{target.device}1") 
		sys_rep_res("umount /dev/mapper/#{target.device}p1") 
		sys_rep_res("sync")
		sleep 5
		sys_rep_res("umount /dev/#{target.device}1") 
		sys_rep_res("umount /dev/mapper/#{target.device}p1") 
		sleep 2 
		sys_rep_res("umount -f /dev/#{target.device}1") 
		sys_rep_res("umount -f /dev/mapper/#{target.device}p1") 
	end
	if sys_rep_res("mountpoint /desinfect/target") 
		# puts "Installation fehlgeschlagen... Data-Partition konnte nicht ausgehängt werden!"  
		error_dialog(get_trans("error_restart"), get_trans("part4") )
		exit 1
	end
	# Install base system
	install_running = true
	update_pgbar(pgbar, get_trans("part5") )
	tgtdev = "/dev/#{target.device}2"
	tgtdev = "/dev/mapper/#{target.device}p2" if target.device =~ /^loop/ 
	unless sys_rep_res("mount -t vfat #{tgtdev} /desinfect/target")
		sleep 2
		sys_rep_res("umount -f #{tgtdev}")
		sys_rep_res("mount -t vfat #{tgtdev} /desinfect/target")
	end
	unless sys_rep_res("mountpoint /desinfect/target")
		# puts "Installation fehlgeschlagen... System-Partition konnte nicht eingehängt werden!"  
		error_dialog(get_trans("error_restart") , get_trans("part6") )
		exit 1
	end
	tgtdev = "/dev/#{target.device}5"
	tgtdev = "/dev/mapper/#{target.device}p5" if target.device =~ /^loop/ 
	if gptprefered == true && use_btrfs == true
		sys_rep_res("mkdir -p /desinfect/btrfs")
		sys_rep_res("mount -t btrfs -o rw,compress=lzo #{tgtdev} /desinfect/btrfs")
	end
	copy_term = Vte::Terminal.new
	copy_term.fork_command("bash", [ "bash", "copy.sh"] )
	copy_term.signal_connect("child-exited") { install_running = false } 
	# sys_rep_res("tar -C /cdrom -cf - casper boot isolinux | tar -C /desinfect/target -xvf - ") 
	while install_running == true 
		sleep 0.5
		pgbar.pulse
		while (Gtk.events_pending?)
			Gtk.main_iteration
		end
	end
	if sys_rep_res("mountpoint /desinfect/btrfs") 
		sys_rep_res("btrfs subvolume snapshot /desinfect/btrfs /desinfect/btrfs/desinfect-initial")
		sys_rep_res("umount /desinfect/btrfs") 
	end
	update_pgbar(pgbar, get_trans("part7") )
	# Install syslinux 
	sys_rep_res("cp /desinfect/target/isolinux/isolinux.cfg /desinfect/target/isolinux/syslinux.cfg")
	if File.exist?('/desinfect/target/isolinux/os.cfg.orig')
		sys_rep_res("cp -v /desinfect/target/isolinux/os.cfg.orig /desinfect/target/isolinux/os.cfg")
	else
		sys_rep_res("cp -v /desinfect/target/isolinux/os.cfg /desinfect/target/isolinux/os.cfg.orig")
	end
	if easymode == true
		sys_rep_res('sed -i \'s/label desinfect-easy/label desinfect-easy\n  menu default/g\' /desinfect/target/isolinux/os.cfg')  
		# sys_rep_res('sed -i \'s/set default=0/set default=1/g\' /desinfect/target/boot/grub/grub.cfg') 
	end
	sys_rep_res("sync") 
	unless sys_rep_res("umount /dev/#{target.device}2") 
		sys_rep_res("umount /dev/mapper/#{target.device}p2") 
		sleep 2
		sys_rep_res("umount -f /dev/#{target.device}2") 
		sys_rep_res("umount -f /dev/mapper/#{target.device}p2") 
	end
	1.upto(5) { |n| 
		unless sys_rep_res("umount /dev/#{target.device}#{n}")
			sys_rep_res("umount /dev/mapper/#{target.device}p#{n}")
			sleep 1
			sys_rep_res("umount -f /dev/#{target.device}#{n}")
			sys_rep_res("umount -f /dev/mapper/#{target.device}p#{n}")
		end
	}  
	if sys_rep_res("mountpoint /desinfect/target")
		error_dialog(get_trans("errorrestart"), get_trans("part8") )
		# puts "Installation fehlgeschlagen... System-Partition konnte nicht ausgehängt werden!"  
		exit 1
	end
	tgtdev = "/dev/#{target.device}2"
	tgtdev = "/dev/mapper/#{target.device}p2" if target.device =~ /^loop/ 
	sys_rep_res("/opt/desinfect/usb-installer/syslinux --install -d /isolinux #{tgtdev}") 
	if gptprefered == true
		tgtdev = "/dev/#{target.device}4"
		tgtdev = "/dev/mapper/#{target.device}p4" if target.device =~ /^loop/ 
		# Install UEFI boot files
		update_pgbar(pgbar, "Installiere Bootloader (UEFI)...")
		unless sys_rep_res("mount -t vfat #{tgtdev} /desinfect/target")
			sleep 2
			sys_rep_res("mount -t vfat #{tgtdev} /desinfect/target")
		end
		unless sys_rep_res("mountpoint /desinfect/target")
			# puts "Installation fehlgeschlagen... UEFI-Partition konnte nicht eingehängt werden!"  
			error_dialog(get_trans("errorrestart"), get_trans("part9"))
			exit 1
		end
		sys_rep_res("tar -C /cdrom -cf - EFI unused.EFI casper/vmlinuz casper/vmlinuz.64 casper/initrd.lz casper/initrd.str boot | tar -C /desinfect/target -xvf - ")
		if File.directory?("/cdrom/casper/filesystem.dir") 
			sys_rep_res("rsync -avHP /media/desinfEFI/ /desinfect/target/") 
			if File.exists?("/tmp/initialrd.tar") 
				sys_rep_res("tar -C /desinfect/target -xvf /tmp/initialrd.tar")
				sys_rep_res("rm /tmp/initialrd.tar")
			end
		end
		
		sys_rep_res("mv /desinfect/target/unused.EFI /desinfect/target/EFI")
		#if File.exist?('/desinfect/target/boot/grub/grub.cfg.orig')
		#	sys_rep_res("cp -v /desinfect/target/boot/grub/grub.cfg.orig /desinfect/target/boot/grub/grub.cfg")
		#else
		#	sys_rep_res("cp -v /desinfect/target/boot/grub/grub.cfg /desinfect/target/boot/grub/grub.cfg.orig")
		#end
		sys_rep_res("cp -v /desinfect/target/boot/grub/grub.cfg.usb /desinfect/target/boot/grub/grub.cfg")
		if use_btrfs == true
			sys_rep_res('sed -i \'s%boot=casper%boot=casper bootfrom=/dev/disk/by-label/desinfBTRFS%g\' /desinfect/target/boot/grub/grub.cfg') 
		else
			sys_rep_res('sed -i \'s%boot=casper%boot=casper iso-scan/filename=/software/desinfect-' + @thisyear + '.iso%g\' /desinfect/target/boot/grub/grub.cfg') 
		end
		sys_rep_res('sed -i \'s/set default=0/set default=1/g\' /desinfect/target/boot/grub/grub.cfg') if easymode == true
		# Replace UUID in GRUB config
		# sys_rep_res("sed -i 's/TARGETUUID/#{uuid}/g' /desinfect/target/EFI/BOOT/grub/grub.cfg ")
		sys_rep_res("sync") 
		unless sys_rep_res("umount #{tgtdev}")
			sleep 2
			sys_rep_res("umount #{tgtdev}")
		end
		if sys_rep_res("mountpoint /desinfect/target")
			error_dialog(get_trans("errorrestart"), get_trans("part10") )
			# puts "Installation fehlgeschlagen... UEFI-Partition konnte nicht ausgehängt werden!"  
			exit 1
		end
	end
	# Create Swap file
	# Copy signatures
	tgtdev = "/dev/#{target.device}3"
	tgtdev = "/dev/mapper/#{target.device}p3" if target.device =~ /^loop/ 
	update_pgbar(pgbar, get_trans("copysigs") )
	unless sys_rep_res("mount -t ext4 #{tgtdev} /desinfect/target")
		sleep 2
		sys_rep_res("mount -t ext4 #{tgtdev} /desinfect/target")
	end
	unless sys_rep_res("mountpoint /desinfect/target")
		error_dialog(get_trans("errorrestart"), get_trans("part11"))
		# puts "Installation fehlgeschlagen... Signatur-Partition konnte nicht eingehängt werden!"  
		exit 1
	end
	sys_rep_res("mkdir -p /desinfect/target/desinfect-signatures")
	sys_rep_res("mkdir -p /desinfect/target/deb")
	[ "kaspersky", "eset", "avira", "clamav", "sophos-av", "f-secure" ].each { |d|
		sys_rep_res("mkdir -p /desinfect/target/desinfect-signatures/#{d}")
	}
	# Let our script do the real copy job
	siginst_running = true
	sigs_term = Vte::Terminal.new
	sigs_term.fork_command("bash", [ "bash", "copy-sigs.sh"] )
	sigs_term.signal_connect("child-exited") { siginst_running = false } 
	while siginst_running == true 
		sleep 0.5
		pgbar.pulse
		while (Gtk.events_pending?)
			Gtk.main_iteration
		end
	end
	# Use swap file
	sys_rep_res("dd if=/dev/zero bs=1M count=1 seek=#{swapsize.to_s} of=/desinfect/target/swap.img")
	sys_rep_res("mkswap /desinfect/target/swap.img") 
	# Use signatures
	sys_rep_res("sync") 
	unless sys_rep_res("umount #{tgtdev}") 
		sleep 2
		sys_rep_res("umount #{tgtdev}") 
	end
	1.upto(4) { |n| 
		sys_rep_res("umount /dev/#{target.device}#{n}") 
		sys_rep_res("umount /dev/mapper/#{target.device}p#{n}") 
	}
	if sys_rep_res("mountpoint /desinfect/target")
		# puts "Installation fehlgeschlagen... Signatur-Partition konnte nicht ausgehängt werden!"  
		error_dialog(get_trans("errorrestart"), get_trans("part12") )
		exit 1
	end
	# ...or unmount
	retval = false
	if sys_rep_res("mountpoint /opt/desinfect/signatures") || target.device =~ /^loop/ 
		# There already is a stick present, tell the user he can remove the new one
		1.upto(4) { |n| 
			sys_rep_res("umount /dev/#{target.device}#{n}")
			sys_rep_res("umount /dev/mapper/#{target.device}p#{n}") 
		} 
		retval = true
	else
		# There is no stick present, use our stick
		sys_rep_res("mount /dev/#{target.device}3 /opt/desinfect/signatures")
		sys_rep_res("umount /var/kl/bases_rd")
		# FIXME! Umount some some potentially mounted tmpfs directories
		sys_rep_res("/etc/init.d/esets stop")
		sys_rep_res("/etc/init.d/fsaua stop")
		{ "avira" => "/AntiVir", 
		  "clamav" =>  "/var/lib/clamav", 
		  "kaspersky" => "/var/kl/bases_rd", 
		  "eset" => "/var/opt/eset",
		   "sophos-av" => "/opt/sophos-av",
		   "f-secure" => "/var/opt/f-secure" 
		}.each { |k,v| 
			sys_rep_res("mount --bind /opt/desinfect/signatures/desinfect-signatures/#{k} #{v}")
		}
		sys_rep_res("/etc/init.d/esets start")
		sys_rep_res("mkdir -p /tmp/desinfect_logs")
		sys_rep_res("mkdir -p /opt/desinfect/signatures/logs")
		sys_rep_res("mkdir -p /opt/desinfect/signatures/deb")
		sys_rep_res("cp -v /var/cache/apt/archives/desinfect-meta*.deb /opt/desinfect/signatures/deb")
		sys_rep_res("touch /opt/desinfect/signatures/.desinfect" + @thisyear) 
		sys_rep_res("sync") 
		sys_rep_res("mount --bind /opt/desinfect/signatures/logs /tmp/desinfect_logs")
		freeloop = ` losetup -f `.strip
		sys_rep_res("losetup #{freeloop} /opt/desinfect/signatures/swap.img")
		sys_rep_res("swapon #{freeloop}")
		retval = false
	end
	6.downto(0) { |n|
		sys_rep_res("dmsetup remove /dev/mapper/#{target.device}p#{n}") if target.device =~ /^loop/ 
	}
	return retval
end

#### Now create widgets

window = Gtk::Window.new
window.signal_connect("delete_event") {
	puts "delete event occurred"
	false
}

window.signal_connect("destroy") {
	puts "destroy event occurred"
	Gtk.main_quit
}

#### Layout-Tabelle

ltab = Gtk::Table.new(6, 4, false)

#### Header, fat

head = Gtk::Label.new
head.set_markup("<b>" + get_trans("wintitle") + "</b>")
# ltab.attach(head, 0, 4, 0, 1, Gtk::EXPAND|Gtk::FILL, 5, 5) 

#### Descriptive text

desc = Gtk::Label.new
desc.set_markup(get_trans("windesc") )
desc.set_markup(get_trans("betadesc") ) if @keepntfs == true 
desc.wrap = true
desc.width_request = 480
align = Gtk::Alignment.new(0.1, 0.1, 0.9, 0.9)
align.add desc
ltab.attach(align, 0, 4, 1, 2, Gtk::EXPAND|Gtk::FILL, 5, 5) 

#### Dropdown of suitable drives

tlab = Gtk::Label.new(get_trans("target") )
ltab.attach(tlab, 0, 1, 2, 3, Gtk::EXPAND|Gtk::FILL, 5, 5) 

drop = Gtk::ComboBox.new
drop.width_request = 300
ltab.attach(drop, 1, 3, 2, 3, Gtk::EXPAND|Gtk::FILL, 5, 5) 

#### Re-read button

reread = Gtk::Button.new(get_trans("reread") )
ltab.attach(reread, 3, 4, 2, 3, Gtk::EXPAND|Gtk::FILL, 5, 5) 

#### Checkbox for type

clab = Gtk::Label.new(get_trans("compat") )
ltab.attach(clab, 0, 1, 3, 4, Gtk::EXPAND|Gtk::FILL, 5, 5) 
gptcheck = Gtk::CheckButton.new(get_trans("compat2") )
gptcheck.active = true # if ARGV[0].to_s == "gpt"
# gptcheck.sensitive = false if @keepntfs == true 
ltab.attach(gptcheck, 1, 4, 3, 4, Gtk::EXPAND|Gtk::FILL, 5, 5) 

#### Checkbox for Easy Mode

elab = Gtk::Label.new(get_trans("easyscan") )
ltab.attach(elab, 0, 1, 4, 5, Gtk::EXPAND|Gtk::FILL, 5, 5) 
easycheck = Gtk::CheckButton.new(get_trans("autostd") )
easycheck.active = false
ltab.attach(easycheck, 1, 4, 4, 5, Gtk::EXPAND|Gtk::FILL, 5, 5) 

#### Checkbox for BTRFS

fslab = Gtk::Label.new(get_trans("filesystem") )
ltab.attach(fslab, 0, 1, 5, 6, Gtk::EXPAND|Gtk::FILL, 5, 5) 
btrfscheck = Gtk::CheckButton.new(get_trans("use_btrfs_as_default") )
btrfscheck.active = false
ltab.attach(btrfscheck, 1, 4, 5, 6, Gtk::EXPAND|Gtk::FILL, 5, 5) 
btrfscheck.signal_connect("clicked") {
	if btrfscheck.active?
		error_dialog("Information", get_trans("warn_btrfs") )
	end
}

#### Install button

fillup = Gtk::Alignment.new(0, 0, 1, 1)
fillup.width_request = 180
instbutton = Gtk::Button.new(Gtk::Stock::APPLY)
ltab.attach(fillup, 1, 2, 6, 7, Gtk::EXPAND|Gtk::FILL, 5, 5) 
ltab.attach(instbutton, 2, 3, 6, 7, Gtk::EXPAND|Gtk::FILL, 5, 5) 

#### Cancel Button

cancel = Gtk::Button.new(Gtk::Stock::CANCEL)
ltab.attach(cancel, 3, 4, 6, 7, Gtk::EXPAND|Gtk::FILL, 5, 5) 
cancel.signal_connect("clicked") { Gtk.main_quit } 

fill_drive_combo(drop, instbutton)
if File.directory?("/cdrom/casper/filesystem.dir") 
	btrfscheck.sensitive = false
	gptcheck.sensitive = false 
end

#### Wire some functions

reread.signal_connect("clicked") { fill_drive_combo(drop, instbutton) } 
instbutton.signal_connect("clicked")  { 
	run_installation = false
	if @drivelist[drop.active].size < 15_000_000_000 && btrfscheck.active? == true
		error_dialog("Error", get_trans("btrfs_needs_16g") )
	elsif btrfscheck.active? == true && gptcheck.active? == false
		error_dialog("Error", get_trans("btrfs_requires_gpt") )
	else
		run_installation = confirm_dialog(@drivelist[drop.active])
	end
	if run_installation == true && @drivelist[drop.active].size > 55_000_000_000 && @keepntfs == false
		run_installation = confirm_again_dialog(@drivelist[drop.active])
	end
	if run_installation == true
		install_progress(@drivelist[drop.active], gptcheck.active?, easycheck.active?, btrfscheck.active? )
		Gtk.main_quit
	end
} 

#### 

if @keepntfs == true
	# FIXME: reintroduce this feature...
	exit 1 
	error_dialog(get_trans("note") , get_trans("betadesc2") )
	error_dialog(get_trans("note"), get_trans("resizesync") )
end


window.border_width = 10
window.set_title(get_trans("wintitle") )
window.window_position = Gtk::Window::POS_CENTER_ALWAYS
ltab.row_spacings = 10
window.add ltab
window.show_all

Gtk.main

